package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.dto.CloudMarkingScoreDto;
import com.qmth.eds.bean.dto.ExamSyncStudentDto;
import com.qmth.eds.bean.dto.ScoreDownloadDto;
import com.qmth.eds.bean.result.StudentMarkResult;
import com.qmth.eds.common.entity.CloudMarkingScore;
import com.qmth.eds.common.entity.ExamCourseMapping;
import com.qmth.eds.common.entity.SysUser;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.CloudMarkingScoreMapper;
import com.qmth.eds.service.CloudMarkingScoreService;
import com.qmth.eds.service.ExamCourseMappingService;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class CloudMarkingScoreServiceImpl extends ServiceImpl<CloudMarkingScoreMapper, CloudMarkingScore> implements CloudMarkingScoreService {

    @Resource
    ExamCourseMappingService examCourseMappingService;
    @Resource
    CloudMarkingScoreMapper cloudMarkingScoreMapper;

    @Override
    public void deleteByKeys(Long schoolId, Long semesterId, Long examTypeId, Integer examId) {
        UpdateWrapper<CloudMarkingScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(CloudMarkingScore::getSchoolId, schoolId)
                .eq(CloudMarkingScore::getSemesterId, semesterId)
                .eq(CloudMarkingScore::getExamTypeId, examTypeId)
                .eq(CloudMarkingScore::getExamId, examId);
        this.remove(updateWrapper);
    }

    @Override
    public IPage<CloudMarkingScoreDto> pageData(Long collegeId, Long semesterId, Long examTypeId, String examId, Integer pageNumber, Integer pageSize) {
        Page<CloudMarkingScoreDto> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.pageData(page, collegeId, semesterId, examTypeId, examId);
    }

    @Override
    public List<ExamSyncStudentDto> listExamSyncStudentDtos(Long schoolId, Long semesterId, Long examTypeId, List<String> courses, String openCollege) {
        return this.baseMapper.listExamSyncStudentDtos(schoolId, semesterId, examTypeId, courses, openCollege);
    }

    /**
     * 查询学生成绩
     *
     * @param semesterId
     * @param examTypeId
     * @param openCollege
     * @param courseCode
     * @return
     */
    @Override
    public List<StudentMarkResult> queryStudentMark(Long semesterId, Long examTypeId, Long collegeId, String openCollege, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        // 查询关联课程
        List<ExamCourseMapping> examCourseMappings = examCourseMappingService.listBySemesterIdAndExamTypeIdAndCloudMarkCourseCode(schoolId, semesterId, examTypeId, courseCode);

        List<String> courses = Arrays.asList(courseCode.split(","));
        if (!CollectionUtils.isEmpty(examCourseMappings)) {
            courses = examCourseMappings.stream().map(ExamCourseMapping::getSyncCourseCode).collect(Collectors.toList());
        }
        return cloudMarkingScoreMapper.queryStudentMark(schoolId, collegeId, semesterId, examTypeId, openCollege, courses);
    }

    @Override
    public List<ScoreDownloadDto> listScoreBySemesterIdAndExamTypeIdAndCollegeIdAndCourseCodeAndOpenCollege(Long semesterId, Long examTypeId, Long collegeId, String courseCode, String openCollege) {
        return this.baseMapper.listScoreBySemesterIdAndExamTypeIdAndCollegeIdAndCourseCodeAndOpenCollege(semesterId, examTypeId, collegeId, courseCode, openCollege);
    }
}
